//Graphics_Functions.c

#include "Graphics_Functions.h"
#include "globals.h"
#include "rs232.h"
#include <string.h>

int packet[] = {0,0,0,0,0,0,0,0,0,0}; 
int instruction[] = {0,0,0,0,0,0,0,0};

int frame_sel = 0;
int draw_mode = M16bit_SINGLE_320x240;


void fillRectangle(int x, int y, int w, int h, int r, int g, int b)
{
	instruction[0] = SQUARE_FILL;
	instruction[1] = r*256 + g*16 + b;
	instruction[2] = x;
	instruction[3] = y;
	instruction[4] = w;
	instruction[5] = h;
	instruction[6] = (draw_mode << 5) + (frame_sel << 3);
	instruction[7] = 0;
	
	sendInstruction(instruction);
}


void fillCircle(int x, int y, int r, int red, int green, int blue)
{
	instruction[0] = CIRCLE_FILL;
	instruction[1] = red*256 + green*16 + blue;
	instruction[2] = x;
	instruction[3] = y;
	instruction[4] = r;
	instruction[5] = 0;
	instruction[6] = (draw_mode << 5) + (frame_sel << 3);
	instruction[7] = 0;
	
	sendInstruction(instruction);
}


void drawRectangle(int x, int y, int w, int h, int r, int g, int b)
{
	instruction[0] = SQUARE_DRAW;
	instruction[1] = r*256 + g*16 + b;
	instruction[2] = x;
	instruction[3] = y;
	instruction[4] = w;
	instruction[5] = h;
	instruction[6] = (draw_mode << 5) + (frame_sel << 3);
	instruction[7] = 0;
	
	sendInstruction(instruction);
}


void drawCircle(int x, int y, int r, int red, int green, int blue)
{
	instruction[0] = CIRCLE_DRAW;
	instruction[1] = red*256 + green*16 + blue;
	instruction[2] = x;
	instruction[3] = y;
	instruction[4] = r;
	instruction[5] = 0;
	instruction[6] = (draw_mode << 5) + (frame_sel << 3);
	instruction[7] = 0;
	
	sendInstruction(instruction);
}


void drawLine(int x1, int x2, int y1, int y2, int r, int g, int b)
{
	instruction[0] = LINE_DRAW;
	instruction[1] = r*256 + g*16 + b;
	instruction[2] = x1;
	instruction[3] = y1;
	instruction[4] = x2;
	instruction[5] = y2;
	instruction[6] = (draw_mode << 5) + (frame_sel << 3);
	instruction[7] = 0;
	
	sendInstruction(instruction);
}

void drawPolygon(int *x, int *y, int elems, int r, int g, int b)
{
	int i;
	for(i = 1; i < elems; i ++)
	{
		drawLine(x[i],x[i-1],y[i],y[i-1], r, g, b);
		printf("drawing line from (%d,%d) to (%d,%d)\n",x[i],y[i],x[i-1],y[i-1]);
	}
	if(elems >= 2)
		drawLine(x[elems-1],x[0],y[elems-1],y[0], r, g, b);
	printf("drawing line from (%i,%i) to (%i,%i)\n",x[elems-1],y[elems-1],x[0],y[0]);
}

void drawChar(int x1, int y1, int charVal, int r, int g, int b)
{
	instruction[0] = TEXT_WRITE;
	instruction[1] = r*256 + g*16 + b;
	instruction[2] = x1;
	instruction[3] = y1;
	instruction[4] = charVal >> 2;
	instruction[5] = (charVal & 0x3) << 2;
	instruction[6] = (draw_mode << 5) + (frame_sel << 3);
	instruction[7] = 0;
	
	sendInstruction(instruction);
}

void writeInstruction(int PC, char a, char b, char c, char d, char e, char f)
{
	instruction[0] = a;
	instruction[1] = b;
	instruction[2] = c;
	instruction[3] = d;
	instruction[4] = e;
	instruction[5] = f;
	instruction[6] = PC >> 4;
	instruction[7] = (((PC&0xF) << 4) | 0x0F);
	sendInstruction(instruction);
}

void startProgram(void)
{
	int i;
	for(i = 0; i < 8; i ++) instruction[i] = 0xFF;
	sendInstruction(instruction);
}
void startDownload(void)
{
	instruction[0] = START_DOWNLOAD;
	sendInstruction(instruction);
}
void finishDownload(void)
{
	instruction[0] = FINISH_DOWNLOAD;
	sendInstruction(instruction);
}

void frameUpdate(void)
{
	instruction[0] = FRAME_SWITCH;
	sendInstruction(instruction);
	switch(draw_mode)
	{
		case M16bit_SINGLE_320x240:
			frame_sel = 0;
			break;
		case M8bit_SINGLE_640x480:
			frame_sel = 0;
			break;
		case M8bit_DOUBLE_320x240:
			frame_sel = (frame_sel == 0) ? 1 : 0;
			break;
		case M4bit_DOUBLE_640x480:
			frame_sel = (frame_sel == 0) ? 1 : 0;
			break;
	}
}

void setDrawMode(int dm)
{
	draw_mode = dm;
	instruction[0] = DRAW_MODE;
	instruction[6] = (draw_mode << 5) + (frame_sel << 3);
	//sendInstruction(instruction);
}

void drawString(int x1, int y1, char *s, int r, int g, int b)
{
	int i = 0;
	for(i = 0; i < strlen(s); i ++)
	{
		int charVal = s[i]*8 - 8*16;
		
		drawChar(x1+i*8,y1,charVal,r,g,b);
	}
}



void sendInstruction(int *instr)
{
	unsigned char result;
	
	int tries = 0;		
	
	//int gray = 1;
	//while(gray == 1)
	//	gray = PollComport(0,&result,1); //to flush out initial data
		
	//do
	//printf("********::  %i   ::********\n",instr[4]);
	int i;
	{
		result = 0;
		packet[0] = ((instr[0] & 0x3F) << 2) | ((instr[1] & 0xC00) >> 10);
		packet[1] = ( instr[1] & 0x3FC) >> 2;
		packet[2] = ((instr[1] & 0x3) << 6) | ((instr[2] & 0x3F0) >> 4);
		packet[3] = ((instr[2] & 0xF) << 4) | ((instr[3] & 0x1E0) >> 5);
		packet[4] = ((instr[3] & 0x1F) << 3) | ((instr[4] & 0x380) >> 7);
		packet[5] = ((instr[4] & 0x7F) << 1) | ((instr[5] & 0x100) >> 8);
		packet[6] = ((instr[5] & 0xFF));
		packet[7] = instr[6] & 0xFF;
		packet[8] = instr[7] & 0xFF;
		packet[9] = 0;
		
		
		for(i = 0; i < (PACKET_LENGTH-1); i ++)
		{
			packet[PACKET_LENGTH-1] ^= packet[i];
			SendByte(0,(unsigned char)packet[i]);
			//printf("%i: %i\n",i,packet[i]);
		}
		
		SendByte(0,(unsigned char)packet[PACKET_LENGTH-1]);
		
		tries ++;
		int pollResult = 0;
		while(pollResult == 0){
			pollResult =PollComport(0,&result,1);
			//printf(".");
		}
		//printf("Got Ack: %d\n",result);
		//printf("tries: %d   result = %d, poll_res = %i\n",tries, result,pollResult);
		
	}
	/*int wait = 0;
	for(i = 0; i < 10000000; i ++)
		wait ++;
	i = wait;*/
	//while(result < 128);
	
	
	
}


